/*****************************************************************************
  BH1749NUC.cpp

 Copyright (c) 2022 ROHM Co.,Ltd.

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MEresultHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
******************************************************************************/
#include <Arduino.h>
#include <Wire.h>
#include <BH1749NUC.h>

BH1749NUC::BH1749NUC(bool addr_pin)
{
    if (addr_pin) {
        Address = BH1749NUC_SLAVE_ADDRESS_39;
    } else {
        Address = BH1749NUC_SLAVE_ADDRESS_38;
    }

    return;
}

BH1749NUC::~BH1749NUC()
{
    Address = 0;

    return;
}

int32_t BH1749NUC::init(void)
{
    int32_t result;

    result = init_check_val();
    if (result == BH1749NUC_COMM_OK) {
        (void)init_setting_ic();
    }

    return result;
}


int32_t BH1749NUC::start(void)
{
    int32_t result;
    uint8_t val = BH1749NUC_CNTL2_RGB_EN;

    result = write(BH1749NUC_MODE_CONTROL2, &val, sizeof(val));

    return result;
}

int32_t BH1749NUC::stop(void)
{
    int32_t result;
    uint8_t val = ~BH1749NUC_CNTL2_RGB_EN;

    result = write(BH1749NUC_MODE_CONTROL2, &val, sizeof(val));

    return result;
}

int32_t BH1749NUC::get_val(uint8_t *raw)
{
    int32_t result;

    result = read(BH1749NUC_RED_DATA_LSB, raw, BH1749NUC_RED_DATA_SIZE);

    return result;
}

int32_t BH1749NUC::get_val(uint16_t *data)
{
    int32_t result;
    uint8_t raw[BH1749NUC_RED_DATA_SIZE];

    result = get_val(raw);
    if (result == BH1749NUC_COMM_OK) {
        //raw[6] and raw[7] are RESERVED Register Value
        data[0] = ((uint16_t)raw[1] << 8)  | raw[0];
        data[1] = ((uint16_t)raw[3] << 8)  | raw[2];
        data[2] = ((uint16_t)raw[5] << 8)  | raw[4];
        data[3] = ((uint16_t)raw[9] << 8)  | raw[8];
        data[4] = ((uint16_t)raw[11] << 8) | raw[10];
    }

    return result;
}

float32 BH1749NUC::get_version(void)
{
    return BH1749NUC_DRIVER_VERSION;
}

int32_t BH1749NUC::init_check_val(void)
{
    int32_t result;
    uint8_t part_id     = 0;
    uint8_t manufact_id = 0;

    result = read(BH1749NUC_SYSTEM_CONTROL, &part_id, sizeof(part_id));
    (void)read(BH1749NUC_MANUFACTURER_ID, &manufact_id, sizeof(manufact_id));
    if (result == BH1749NUC_COMM_OK) {
        part_id      &= BH1749NUC_PART_ID_MASK;
        manufact_id  &= BH1749NUC_MANUFACTURER_ID_MASK;

        if ((part_id == BH1749NUC_PART_ID_VAL) && (manufact_id == BH1749NUC_MANUFACTURER_ID_VAL)) {
            result = BH1749NUC_COMM_OK;
        } else {
            result = BH1749NUC_ID_ERROR;
        }
    } else { 
        result = BH1749NUC_COMM_ERROR;
    }

    return result;
}

int32_t BH1749NUC::init_setting_ic(void)
{
    int32_t result;
    uint8_t cntl = BH1749NUC_MODE_CONTROL1_VAL;

    result = write(BH1749NUC_MODE_CONTROL1, &cntl, sizeof(cntl));
    if (result == BH1749NUC_COMM_OK) {
        // No Process
    } else {
        result = BH1749NUC_COMM_ERROR;
    }

    return result;
}

int32_t BH1749NUC::write(uint8_t reg, uint8_t *data, int32_t size)
{
    int32_t result;

    Wire.beginTransmission(Address);
    (void)Wire.write(reg);
    (void)Wire.write(data, (uint8_t)size);
    result = Wire.endTransmission();
    if (result == 0) {
        result = BH1749NUC_COMM_OK;
    } else {
        result = BH1749NUC_COMM_ERROR;
    }

    return result;
}

int32_t BH1749NUC::read(uint8_t reg, uint8_t *data, int32_t size)
{
    int32_t result;
    uint8_t cnt;
    uint8_t receive_size;

    Wire.beginTransmission(Address);
    (void)Wire.write(reg);
    result = Wire.endTransmission(false);
    if (result == 0) {
        receive_size = Wire.requestFrom(Address, (uint8_t)size);
        if (receive_size == size) {
            for (cnt = 0; cnt < size; cnt++) {
                data[cnt] = Wire.read();
            }
            result = BH1749NUC_COMM_OK;
        } else {
            result = BH1749NUC_COMM_ERROR;
        }
    } else {
        result = BH1749NUC_COMM_ERROR;
    }

    return result;
}